import 'package:flutter/material.dart';
import 'package:yin_ui/actionsheet/utils.dart';
import 'package:yin_ui/theme/index.dart';

class AndroidWidget extends StatefulWidget {
  //蒙版可否点击
  final bool maskClosable;

  //关闭
  final Function() close;

  //变化
  final Function(int index) onChange;

  //子视图
  final List<dynamic> options;

  AndroidWidget({
    key,
    this.maskClosable,
    this.close,
    this.onChange,
    this.options,
  }) : super(key: key);

  @override
  AndroidWidgetState createState() => new AndroidWidgetState();
}

class AndroidWidgetState extends State<AndroidWidget> with SingleTickerProviderStateMixin {
  AnimationController _controller;
  Animation<double> _animation;
  int _index;
  WeTheme theme;
  double _leftPadding = 35.0;

  @override
  void initState() {
    super.initState();

    _controller = AnimationController(vsync: this, duration: Duration(milliseconds: 200));
    _animation = Tween(begin: 0.0, end: 1.0).animate(_controller)
      ..addListener(() {
        setState(() {});
      })
      ..addStatusListener(animateListener);

    //播放
    _controller.forward();
  }

  @override
  Widget build(BuildContext context) {
    return Opacity(
      opacity: _animation.value,
      child: GestureDetector(
        onTap: () {
          if (widget.maskClosable) maskClick();
        },
        child: Container(
          decoration: BoxDecoration(color: theme.maskColor),
          alignment: Alignment.center,
          padding: EdgeInsets.only(left: _leftPadding, right: _leftPadding),
          child: Material(
            color: Colors.white,
            shadowColor: Colors.red,
            borderRadius: BorderRadius.all(Radius.circular(2)),
            child: ConstrainedBox(
              constraints: BoxConstraints(
                maxHeight: MediaQuery.of(context).size.height * 0.8,
              ),
              child: SingleChildScrollView(
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: initChilder(
                    widget.options,
                    itemClick,
                    theme.defaultBorderColor,
                    align: Alignment.centerLeft,
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    theme = WeUI.getTheme(context);
  }

  //动画监听
  animateListener(state) {
    if (state == AnimationStatus.dismissed) {
      if (_index == null) {
        widget.close();
      } else {
        widget.onChange(_index);
      }
    }
  }

  //遮层点击
  void maskClick() {
    _index = null;
    _controller.reverse();
  }

  //item点击
  void itemClick(int index) {
    _index = index;
    _controller.reverse();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }
}
