//关闭函数
import 'package:flutter/material.dart';
import 'package:yin_ui/actionsheet/android_widget.dart';
import 'package:yin_ui/actionsheet/ios_widget.dart';
import 'package:yin_ui/utils.dart';

typedef _Close = Function();

typedef _onChange = Function(String inde);

//安卓
typedef WeActionSheetAndroid = Function({
  @required List<WeActionSheetItem> options,
  _Close onClose,
  bool maskClosable,
  _onChange onChange,
});

//ios
typedef WeActionSheetIos = Function({
  dynamic title,
  @required List<WeActionSheetItem> options,
  bool maskClosable,
  dynamic cancelButton,
  _Close onClose,
  _onChange onChange,
});

class WeActionSheet {
  //android 样式
  static WeActionSheetAndroid android(BuildContext context) {
    return ({
      maskClosable = true,
      onClose,
      onChange,
      options,
    }) {
      final GlobalKey widgetKey = GlobalKey();
      Function remove;

      //关闭
      void hide() {
        remove();
        if (onClose is Function) onClose();
      }

      //item选择
      void itemClick(int index) {
        remove();
        if (onChange is Function) {
          final String value = options[index].value;
          onChange(value ?? index.toString());
        }
      }

      remove = createOverlayEntry(
        context: context,
        child: AndroidWidget(
          key: widgetKey,
          maskClosable: maskClosable,
          close: hide,
          onChange: itemClick,
          options: options,
        ),
        willPopCallback: () {
          (widgetKey.currentState as AndroidWidgetState).maskClick();
        },
      );
    };
  }

  // ios 样式
  static WeActionSheetIos ios(BuildContext context) {
    return ({
      title,
      options,
      maskClosable = true,
      cancelButton,
      onClose,
      onChange,
    }) {
      final GlobalKey widgetKey = GlobalKey();
      Function remove;

      // 关闭
      void hide() {
        remove();
        if (onClose is Function) {
          onClose();
        }
      }

      // 点击
      void itemClick(int index) {
        remove();
        if (onChange is Function) {
          final String value = options[index].value;
          onChange(value == null ? index.toString() : value);
        }
      }

      remove = createOverlayEntry(
        context: context,
        child: IosWidget(
          key: widgetKey,
          title: title,
          cancelButton: cancelButton,
          maskClosable: maskClosable,
          close: hide,
          onChange: itemClick,
          options: options,
        ),
        willPopCallback: () {
          (widgetKey.currentState as IosWidgetState).close();
        },
      );
    };
  }
}

class WeActionSheetItem {
  final label;
  final String value;

  WeActionSheetItem({this.label, this.value});
}
