//弹框类型
import 'package:flutter/material.dart';
import 'package:yin_ui/dialog/custom_widget.dart';
import 'package:yin_ui/utils.dart';

import 'ios_widget.dart';
import 'android_widget.dart';
import 'service_widget.dart';

//平台风格
enum WeDialogPlatform { android, ios }

//实现方式
enum WeDialogModel { overlay, dialog }

//显示类型
enum WeDialogType { alert, confirm, custom, service }

//弹框
typedef Alert = Function(
  dynamic message, {
  //标题
  dynamic title,
  //平台风格
  WeDialogPlatform platform,
  // 对话框样式
  WeDialogModel model,
  //按钮
  dynamic button,
  //按钮事件
  Function onConfirm,
  //点击其他区域关闭对话框
  bool barrierDismissible,
});

//确认框
typedef Confirm = Function(
  dynamic message, {
  //标题
  dynamic title,
  //平台风格
  WeDialogPlatform platform,
  // 对话框样式
  WeDialogModel model,
  //取消按钮
  dynamic cancelButton,
  //确定按钮
  dynamic confirmButton,
  //确定事件
  Function onConfirm,
  //取消事件
  Function onCancel,
  //点击其他区域关闭对话框
  bool barrierDismissible,
});

//自定义框
typedef Custom = Function(
  dynamic message, {
  // 对话框样式
  WeDialogModel model,
  //点击其他区域关闭对话框
  bool barrierDismissible,
});

//服务条款阅读框
typedef Service = Function(
  dynamic message, {
  //标题
  dynamic title,
  //确定按钮
  dynamic confirmButton,
  // 对话框样式
  WeDialogModel model,
  //确定事件
  Function onConfirm,
  //持续时间
  int duration,
  // 倒计时显示
  dynamic otherButton,
});

Function _createAndroidLayout(
  BuildContext context,
  message, {
  //标题
  dynamic title,
  //平台风格
  WeDialogPlatform platform,
  //显示类型
  WeDialogModel model,
  //对话框类型
  WeDialogType dialogType,
  //取消按钮
  dynamic cancelButton,
  //确定按钮
  dynamic confirmButton,
  //取消事件
  Function onCancel,
  //确定事件
  Function onConfirm,
  // 对话框外操作可以关闭对话
  bool barrierDismissible,
  //持续时间
  int duration,
  // 倒计时显示文字
  dynamic otherButton,
}) {
  Function remove;

  final GlobalKey globalKey = GlobalKey();

  Widget widget;

  void close() async {
    if (dialogType == WeDialogType.alert || dialogType == WeDialogType.confirm) {
      if (platform == WeDialogPlatform.android) {
        await (globalKey.currentState as AndroidWidgetState).reverseAnimation();
      } else if (platform == WeDialogPlatform.ios) {
        await (globalKey.currentState as IosWidgetState).reverseAnimation();
      }
    } else if (dialogType == WeDialogType.service) {
      await (globalKey.currentState as ServiceWidgetState).reverseAnimation();
    } else if (dialogType == WeDialogType.custom) {
      await (globalKey.currentState as CustomWidgetState).reverseAnimation();
    }
    remove();
  }

  //确认
  void confirm() {
    close();
    Future.delayed(Duration(milliseconds: 150), () {
      if (onConfirm is Function) onConfirm();
    });
  }

  //取消
  void cancel() {
    close();
    Future.delayed(Duration(milliseconds: 150), () {
      if (onCancel is Function) onCancel();
    });
  }

  //蒙版点击与物理返回键
  void other() {
    if (!barrierDismissible) return;
    close();
  }

  final List<Map<String, Object>> buttons = [
    {
      "widget": toTextWidget(confirmButton, 'button'),
      'onClick': confirm,
    }
  ];

  if (dialogType == WeDialogType.confirm) {
    buttons.insert(0, {
      "widget": toTextWidget(cancelButton, 'button'),
      'onClick': cancel,
    });
  }
  if (dialogType == WeDialogType.service) {
    widget = ServiceWidget(
      key: globalKey,
      title: toTextWidget(title, 'title'),
      message: toTextWidget(message, 'message'),
      buttons: buttons,
      onMaskCLick: other,
      duration: duration,
      otherButton: otherButton,
      model: model,
    );
  } else if (dialogType == WeDialogType.custom) {
    widget = CustomWidget(
      key: globalKey,
      child: toTextWidget(message, 'message'),
      onMaskCLick: other,
      model: model,
    );
  } else if (platform == WeDialogPlatform.android) {
    widget = AndroidWidget(
      key: globalKey,
      title: toTextWidget(title, 'title'),
      message: toTextWidget(message, 'message'),
      buttons: buttons,
      onMaskCLick: other,
      model: model,
    );
  } else {
    widget = IosWidget(
      key: globalKey,
      title: toTextWidget(title, 'title'),
      message: toTextWidget(message, 'message'),
      buttons: buttons,
      onMaskCLick: other,
      model: model,
    );
  }

  if (model == WeDialogModel.overlay) {
    remove = createOverlayEntry(
      context: context,
      child: widget,
      willPopCallback: other,
    );
  } else if (model == WeDialogModel.dialog) {
    remove = createDialog(
      context: context,
      child: widget,
      willPopCallback: other,
      barrierDismissible: barrierDismissible,
    );
  }

  return close;
}

///对话框
class WeDialog {
  //提示框
  static Alert alert(BuildContext context) {
    return (
      message, {
      title = '提示',
      platform = WeDialogPlatform.ios,
      model = WeDialogModel.dialog,
      button = '确认',
      onConfirm,
      barrierDismissible = true,
    }) {
      return _createAndroidLayout(
        context,
        message,
        title: title,
        platform: platform,
        model: model,
        dialogType: WeDialogType.alert,
        confirmButton: button,
        onConfirm: onConfirm,
        barrierDismissible: barrierDismissible,
      );
    };
  }

  //确认对话框
  static Confirm confirm(BuildContext context) {
    return (
      message, {
      title = '提示',
      platform = WeDialogPlatform.ios,
      model = WeDialogModel.dialog,
      cancelButton = '取消',
      onCancel,
      confirmButton = '确认',
      onConfirm,
      barrierDismissible = true,
    }) {
      return _createAndroidLayout(
        context,
        message,
        title: title,
        platform: platform,
        model: model,
        dialogType: WeDialogType.confirm,
        confirmButton: confirmButton,
        onConfirm: onConfirm,
        cancelButton: cancelButton,
        onCancel: onCancel,
        barrierDismissible: barrierDismissible,
      );
    };
  }

  //自定义视图
  static Custom custom(BuildContext context) {
    return (
      message, {
      model = WeDialogModel.dialog,
      barrierDismissible = true,
    }) {
      return _createAndroidLayout(
        context,
        message,
        model: model,
        dialogType: WeDialogType.custom,
        barrierDismissible: barrierDismissible,
      );
    };
  }

  //服务条款阅读
  static Service service(BuildContext context) {
    return (
      message, {
      title = '提示',
      model = WeDialogModel.dialog,
      confirmButton = '确认',
      onConfirm,
      duration = 3,
      otherButton = '后可关闭',
    }) {
      return _createAndroidLayout(
        context,
        message,
        title: title,
        model: model,
        dialogType: WeDialogType.service,
        confirmButton: confirmButton,
        onConfirm: onConfirm,
        barrierDismissible: false,
        duration: duration,
        otherButton: otherButton,
      );
    };
  }
}
