import 'package:flutter/material.dart';

import 'package:yin_ui/animation/fade_in.dart';

//图片类型
enum PreviewType { network, local, custom }

///图片展示
class Preview extends StatefulWidget {
  //图片列表
  final List<String> images;

  //背景图片
  final Color color;

  //遮层点击
  final Function maskClick;

  //默认展示
  final int defaultIndex;

  //指示显示
  final bool indicators;

  //指示颜色
  final Color indicatorsColor;

  //图片类型
  final PreviewType type;

  //自定义视图
  final Function(BuildContext context, int index) itemBuild;

  Preview({
    key,
    this.images,
    this.color,
    this.maskClick,
    this.defaultIndex,
    this.indicators,
    this.indicatorsColor,
    this.type = PreviewType.network,
    this.itemBuild,
  }) : super(key: key);

  @override
  PreviewState createState() => new PreviewState();
}

class PreviewState extends State<Preview> {
  GlobalKey fadeInKey = GlobalKey();
  PageController _pageController;
  int _index = 0;

  @override
  void initState() {
    super.initState();
    _index = widget.defaultIndex;
    _pageController = PageController(initialPage: widget.defaultIndex);
  }

  //反顺动画
  reverseAnimation() async {
    await (fadeInKey.currentState as FadeInState).controller.reverse();
  }

  //切换事件
  void onChange(int index) {
    _index = index;
    setState(() {});
  }

  //渲染指标
  Widget renderIndicators() {
    final List<Widget> indicators = [];
    final double size = 7.0;
    List.generate(widget.images.length, (index) {
      final child = Padding(
        padding: EdgeInsets.only(left: index == 0 ? 0 : 7),
        child: Opacity(
          opacity: _index == index ? 1 : 0.55,
          child: SizedBox(
            width: size,
            height: size,
            child: DecoratedBox(
              decoration: BoxDecoration(
                color: widget.indicatorsColor,
                borderRadius: BorderRadius.all(Radius.circular(size)),
              ),
            ),
          ),
        ),
      );
      indicators.add(child);
    });

    return Row(
      children: indicators,
      mainAxisAlignment: MainAxisAlignment.center,
    );
  }

  //渲染item
  Widget renderItemView(int index) {
    Widget child;
    if (widget.type == PreviewType.local) {
      child = Image.asset(widget.images[index]);
    } else if (widget.type == PreviewType.custom) {
      child = widget.itemBuild(context, index);
    } else {
      child = Image.network(widget.images[index]);
    }
    return GestureDetector(
      onTap: widget.maskClick,
      child: DecoratedBox(
        decoration: BoxDecoration(color: Colors.transparent),
        child: Padding(
          padding: EdgeInsets.only(left: 2, right: 2),
          child: Align(child: child),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    Widget child = PageView.builder(
      onPageChanged: onChange,
      controller: _pageController,
      itemCount: widget.images.length,
      itemBuilder: (context, index) => renderItemView(index),
    );

    //判断是否显示指标
    if (widget.indicators) {
      child = Stack(
        children: <Widget>[
          child,
          Positioned(
            right: 0,
            left: 0,
            bottom: 20,
            child: renderIndicators(),
          ),
        ],
      );
    }

    return FadeIn(
      key: fadeInKey,
      duration: 200,
      child: DecoratedBox(
        decoration: BoxDecoration(color: widget.color),
        child: child,
      ),
    );
  }

  @override
  void dispose() {
    _pageController.dispose();
    super.dispose();
  }
}
