import 'package:flutter/material.dart';
import 'package:yin_ui/animation/rotating.dart';
import 'package:yin_ui/utils.dart';

enum PageType {
  loading,
  empty,
  reload,
}

Widget loadIcon = Image.asset(
  'assets/images/loading.png',
  height: 60,
  width: 60,
  package: 'yin_ui',
);

Widget emptyIcon = Icon(Icons.hourglass_empty, size: 60, color: Colors.grey);

Widget errorIcon = Icon(Icons.error_outline, size: 60, color: Colors.grey);

Widget _sizeBox = SizedBox(width: 20, height: 20);

///页面加载
class LoadEmptyError extends StatelessWidget {
  final PageType type;
  final dynamic loadWidget;
  final dynamic emptyWidget;
  final dynamic reloadWidget;
  final Function reloadClick;

  LoadEmptyError({
    this.type = PageType.loading,
    this.loadWidget = '加载中...',
    this.emptyWidget = '无数据',
    this.reloadWidget = '数据获取失败',
    this.reloadClick,
  });

  @override
  Widget build(BuildContext context) {
    List<Widget> loadChils = [
      _sizeBox,
      DefaultTextStyle(
        style: TextStyle(fontSize: 16, color: Colors.black87),
        child: toTextWidget(loadWidget, 'loadWidget'),
      ),
    ];
    List<Widget> emptyChils = [
      _sizeBox,
      DefaultTextStyle(
        style: TextStyle(fontSize: 16, color: Colors.black87),
        child: toTextWidget(emptyWidget, 'emptyWidget'),
      ),
    ];
    List<Widget> reloadChils = [
      _sizeBox,
      DefaultTextStyle(
        style: TextStyle(fontSize: 16, color: Colors.black87),
        child: toTextWidget(reloadWidget, 'reloadWidget'),
      ),
    ];

    final Widget reload = InkWell(
      onTap: buttonClick,
      child: Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.all(Radius.circular(3)),
          color: Colors.green,
        ),
        padding: EdgeInsets.only(left: 10, right: 10, top: 5, bottom: 5),
        child: Text(
          '重新加载',
          style: TextStyle(fontSize: 14, color: Colors.white),
        ),
      ),
    );

    if (loadWidget is String) {
      loadChils.insert(0, Rotating(loadIcon));
    }

    if (emptyWidget is String) {
      emptyChils.insert(0, emptyIcon);
    }

    if (reloadWidget is String) {
      reloadChils.insert(0, errorIcon);
      reloadChils.add(_sizeBox);
      reloadChils.add(reload);
    }

    final Widget loadView = Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: loadChils,
    );
    final Widget emptyView = Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: emptyChils,
    );
    final Widget reloadView = Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: reloadChils,
    );

    Widget child;

    switch (type) {
      case PageType.loading:
        child = loadView;
        break;
      case PageType.empty:
        child = emptyView;
        break;
      case PageType.reload:
        child = reloadView;
        break;
    }

    return SizedBox(
      width: double.infinity,
      child: Padding(
        padding: EdgeInsets.all(16),
        child: child,
      ),
    );
  }

  void buttonClick() {
    if (reloadClick is Function) reloadClick();
  }
}
