import 'dart:ui';

import 'package:flutter/material.dart';

///通知
class NotifyWidget extends StatefulWidget {
  final Widget child;
  final Color color;

  NotifyWidget({
    key,
    this.child,
    this.color = Colors.white,
  }) : super(key: key);

  @override
  NotifyWidgetState createState() => NotifyWidgetState();
}

class NotifyWidgetState extends State<NotifyWidget>
    with TickerProviderStateMixin {
  AnimationController controller;

  Animation<double> animation;

  @override
  void initState() {
    super.initState();

    controller = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 300),
    );

    animation = Tween<double>(begin: -1.0, end: 0.0)
        .animate(CurvedAnimation(parent: controller, curve: Curves.ease))
          ..addListener(() {
            this.setState(() {});
          });

    //动画启动
    controller.forward();
  }

  //动画反向
  reverseAnimation() async {
    await controller.reverse();
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        Positioned(
          top: 0,
          left: 0,
          right: 0,
          child: FractionalTranslation(
            translation: Offset(0, animation.value),
            child: DecoratedBox(
              decoration: BoxDecoration(color: widget.color),
              child: Padding(
                padding: EdgeInsets.only(
                  top: MediaQueryData.fromWindow(window).padding.top + 15,
                  left: 16,
                  right: 16,
                  bottom: 16,
                ),
                child: Align(
                  alignment: Alignment.center,
                  child: DefaultTextStyle(
                    style: TextStyle(fontSize: 14, color: Colors.white),
                    child: widget.child,
                  ),
                ),
              ),
            ),
          ),
        ),
      ],
    );
  }

  @override
  void dispose() {
    controller?.dispose();
    super.dispose();
  }
}
