import 'package:flutter/material.dart';
import 'package:yin_ui/icon/index.dart';
import 'package:yin_ui/tip/index.dart';
import 'dart:math' as math;

//间距
final double spacing = 8.0;
// 背景色
final Color _color = Color(0xff303133);

class WeTipWidget extends StatelessWidget {
  final Function maskClick;
  final Widget child;
  final boxContext;
  final WeTipPlacement placement;
  Offset offset;
  double top;
  double left;
  Offset triangleOffset;
  double triangleTop;
  double triangleLeft;
  double angle;

  WeTipWidget({
    this.maskClick,
    this.child,
    this.boxContext,
    this.placement,
  }) {
    final Size size = boxContext.size;
    final RenderBox box = boxContext.findRenderObject();
    final Offset boxOffset = box.localToGlobal(Offset.zero);

    switch (placement) {
      case WeTipPlacement.top:
        top = boxOffset.dy - spacing;
        left = boxOffset.dx + (size.width / 2);
        offset = Offset(-0.5, -1);
        //三角形
        triangleOffset = offset;
        triangleTop = boxOffset.dy + 2;
        triangleLeft = left;
        angle = math.pi;
        break;
      case WeTipPlacement.bottom:
        top = boxOffset.dy + size.height + spacing;
        left = boxOffset.dx + (size.width / 2);
        offset = Offset(-0.5, 0);
        //三角形
        triangleOffset = offset;
        triangleTop = boxOffset.dy + size.height - 2;
        triangleLeft = left;
        angle = 0;
        break;
      case WeTipPlacement.left:
        top = boxOffset.dy + (size.height / 2);
        left = boxOffset.dx - spacing;
        offset = Offset(-1, -0.5);
        //三角形
        triangleOffset = offset;
        triangleTop = top;
        triangleLeft = boxOffset.dx + 2;
        angle = math.pi / 2;
        break;
      case WeTipPlacement.right:
        top = boxOffset.dy + (size.height / 2);
        left = boxOffset.dx + size.width + spacing;
        offset = Offset(0, -0.5);
        //三角形
        triangleOffset = offset;
        triangleTop = top;
        triangleLeft = boxOffset.dx + size.width - 2;
        angle = (math.pi / 2) * 3;
        break;
    }
  }

  @override
  Widget build(BuildContext context) {
    final Size size = MediaQuery.of(context).size;

    return Stack(
      children: <Widget>[
        //透明图层
        GestureDetector(
          onPanDown: (_) => maskClick(),
          child: DecoratedBox(
            decoration: BoxDecoration(color: Colors.transparent),
            child: SizedBox(
              height: size.height,
              width: size.width,
            ),
          ),
        ),
        //内容视图
        Positioned(
          top: top,
          left: left,
          child: FractionalTranslation(
            translation: offset,
            child: DecoratedBox(
              decoration: BoxDecoration(
                color: _color,
                borderRadius: BorderRadius.all(Radius.circular(4)),
              ),
              child: DefaultTextStyle(
                style: TextStyle(fontSize: 13, color: Colors.white),
                child: Padding(
                  padding: EdgeInsets.all(10),
                  child: child,
                ),
              ),
            ),
          ),
        ),
        //三角形
        Positioned(
          top: triangleTop,
          left: triangleLeft,
          child: FractionalTranslation(
            translation: triangleOffset,
            child: Transform.rotate(
              angle: angle,
              child: Icon(
                WeIcons.triangle,
                color: _color,
                size: 14,
              ),
            ),
          ),
        ),
      ],
    );
  }
}
